@@ -19,6 +19,7 @@
 
 import re
 import os
+import subprocess
 import urlparse
 from urllib2 import urlopen
 import urllib
@@ -64,6 +65,7 @@ if metrics_enabled:
 
 # Lecture des templates de la page d'authentification
 AUTH_FORM = open(os.path.join('interface', 'authform.tmpl')).read()
+CHANGEPWD_FORM = open(os.path.join('interface', 'changepwd.tmpl')).read()
 # SECURID
 if config.USE_SECURID or config.SECURID_PROVIDER:
     from securid_utils import SecuridCheck, SECURID_AUTH_FORM
@@ -174,6 +176,25 @@ class CasResource(Resource):
         if missing:
             raise MissingParameters(missing)
 
+# Support des metrics si mode pushgateway
+class MetricsPushgateway(CasResource):
+    """
+    page de recuperation des metrics depuis pushgateway
+    """
+    addSlash = False
+    content_type = "text/plain; charset=utf-8"
+
+    def __init__(self, manager):
+        self.manager = manager
+
+    def _render(self,request):
+        data="#no pushgateway mode"
+        if self.manager.pushgateway:
+            u=urlopen("http://%s/metrics" % self.manager.pushgateway)
+            data = u.read()
+        response = http.Response(code=responsecode.OK, stream=data)
+        return self.set_headers(response)
+
 class CASCompliantResponse(CasResource):
     """implémentation du protocole CAS pour la validation de ticket
     """
@@ -494,32 +515,32 @@ class LoggedInForm(CasResource):
             css = escape(request.args['css'][0])
         except KeyError:
             css = config.DEFAULT_CSS
-        sso_cookie = getCookie(request, 'EoleSSOServer')
+        sso_cookie = getCookie(request, config.COOKIE_NAME)
+        content = _('Invalid session')
         if sso_cookie:
             user_session = sso_cookie.value
-            sessions = ""
-            local_sessions = []
-            for app_t in self.manager.user_app_tickets[user_session]:
-                if hasattr(app_t, 'provider_data'):
-                    sessions += """<li>Identité fournie par : %s</li>""" % \
-                            self.manager.external_providers[app_t.provider_data['provider']][2]
-                elif hasattr(app_t,'saml_ident'):
-                    sessions += """<li>session distante sur %s</li>""" % app_t.saml_ident
+            if user_session in self.manager.user_sessions:
+                sessions = ""
+                local_sessions = []
+                for app_t in self.manager.user_app_tickets.get(user_session, []):
+                    if hasattr(app_t, 'provider_data'):
+                        sessions += """<li>Identité fournie par : %s</li>""" % \
+                                self.manager.external_providers[app_t.provider_data['provider']][2]
+                    elif hasattr(app_t,'saml_ident'):
+                        sessions += """<li>session distante sur %s</li>""" % app_t.saml_ident
+                    else:
+                        if app_t.service_url not in local_sessions:
+                            local_sessions.append(urllib.quote(app_t.service_url, safe=uri_reserved_chars))
+                if local_sessions:
+                    sessions += "<li>%s</li>" % "</li><li>".join(local_sessions)
+                if sessions:
+                    content = """<h1>Liste de vos sessions ouvertes :</h1><br/>
+                    <ul>%s</ul>""" % sessions
                 else:
-                    if app_t.service_url not in local_sessions:
-                        local_sessions.append(urllib.quote(app_t.service_url, safe=uri_reserved_chars))
-            if local_sessions:
-                sessions += "<li>%s</li>" % "</li><li>".join(local_sessions)
-            if sessions:
-                content = """<h1>Liste de vos sessions ouvertes :</h1><br/>
-                <ul>%s</ul>""" % sessions
-            else:
-                content = """<h1>Aucune session d'application trouvée</h1>"""
-            content += """<form action="/logout" method="post">
-            <p class=formvalidation><input class="btn" type="Submit" value="%s">
-            </p></form>""" % _("Disconnect")
-        else:
-            content = _('Invalid session')
+                    content = """<h1>Aucune session d'application trouvée</h1>"""
+                content += """<form action="logout" method="post">
+                <p class=formvalidation><input class="btn" type="Submit" value="%s">
+                </p></form>""" % _("Disconnect")
         response = http.Response(stream=gen_page(_("Valid SSO session"), content, css))
         return self.set_headers(response)
 
@@ -582,7 +603,7 @@ class LogoutForm(CasResource):
         # invalidation de la session sur le serveur d'authentification
         cookies = getCookies(request)
         for cookie in cookies:
-            if cookie.name == 'EoleSSOServer':
+            if cookie.name == config.COOKIE_NAME:
                 session_id = cookie.value
                 if session_id is not None:
                     # suppression de la session sur le manager
@@ -599,7 +620,7 @@ class LogoutForm(CasResource):
             log.msg(_("Logged out : redirecting to {0}").format(url_from))
         else:
             #si pas de parametre from, on affiche la page de logout en local
-            content = """<form action="/login" method="post">
+            content = """<form action="login" method="post">
             <p class=formvalidation>
             <input class="btn" type="Submit" value="%s"></p></form>""" % _("Connect")
             response = http.Response(stream=gen_page(_('SSO session closed'), content, css))
@@ -691,9 +712,9 @@ class LocalCookie(CasResource):
                 print " not implemented : redirect one level higher !"
             cookies = []
             for cookie in getCookies(request):
-                if cookie.name != 'EoleSSOServer':
+                if cookie.name != config.COOKIE_NAME:
                     cookies.append(cookie)
-            cookies.append(TwCookie("EoleSSOServer", ticket, path = "/", discard=True, secure=True))
+            cookies.append(TwCookie(config.COOKIE_NAME, ticket, domain = config.COOKIE_DOMAIN, path = "/", discard=True, secure=True))
             if cookies:
                 log.msg(_("Local cookie sent, redirecting to child server"))
                 response.headers.setHeader('Set-Cookie', cookies)
@@ -801,7 +822,7 @@ class LoginForm(CasResource):
             if self.manager.get_app_infos(from_url) is None:
                 return '/unauthorizedService?service=%s' % from_url
         # on va verifier si on a un identifiant de session dans le cookie
-        user_session = getCookie(request, 'EoleSSOServer')
+        user_session = getCookie(request, config.COOKIE_NAME)
         # XXX: IE et Opera ne gerent pas les cookies de la meme
         # maniere que Firefox sous firefox, quand un cookie devient
         # invalide (au niveau du temps), il est supprimé sous IE et
@@ -810,7 +831,7 @@ class LoginForm(CasResource):
         # si on en a bien un, on le verifie auprès du serveur d'authentification
         if not renew and user_session and user_session != 'foo':
             session_id = user_session.value.encode(config.encoding)
-            verif =  self.manager.verify_session_id(session_id)
+            verif = self.manager.verify_session_id(session_id)
             # si l'ID de session est verifie, on génère le ticket applicatif,
             # et on redirige vers l'URL de base
             if verif[0]:
@@ -828,7 +849,23 @@ document.location.href="%s";
                         self.content = str("""<html>%s<body onload="setTimeout(function(){RedirectWarn()}, 200);"></body></html>""" % (head % (warn_msg, urllib.quote(url, safe=uri_reserved_chars))))
                         return None
                     return url
-                return '/loggedin'
+                return 'loggedin'
+            else:
+                # Session non valide. On vérifie si la connexion précédente s'est faite par fédération.
+                # Si oui, on redirige vers l'url définie dans assocations.ini pour ce fournisseur (si présente)
+                idp_ident = getCookie(request, '{}_IDP'.format(config.COOKIE_NAME))
+                if idp_ident and idp_ident.value in self.manager.sp_meta:
+                    default_url, force_default = self.manager.get_default_logout_url(idp_ident=idp_ident.value)
+                    if default_url:
+                        logout_status = _('SSO session timed out')
+                        logout_msg = """
+                        <br/><b>{}</b> : <br/><h3><font color="blue">{}</font></h3><br/>
+                        </p><p class="formvalidation">
+                        <button type="button" class="btn" onclick="javascript:location.href='{}'">{}</button></p>
+                        """.format(_('Click here to proceed to authentication service'), default_url, default_url, _('Proceed to above url'))
+                        log.msg('Timeout, Redirect to IDP ({})'.format(default_url))
+                        self.content = gen_page(logout_status, str(logout_msg), css)
+                        return None
 
         if gateway and from_url:
             return urljoin(from_url, 'ticket=')
@@ -845,6 +882,12 @@ document.location.href="%s";
                         self.auth_msg = _("Error occured while authentication to {0}").format(provider_label)
                 else:
                     self.auth_msg = _("Authentication failed, try again")
+        if request_args.has_key('error_ad'):
+            if request_args['error_ad'][0] == '1':
+                self.auth_msg = _("Error connecting AD server")
+        if request_args.has_key('pwdneedchange'):
+            if request_args['pwdneedchange'][0] == '1':
+                self.auth_msg = _("Password expired, please change it")
 
         # création d'un login ticket jouable une seule fois pour cet essai d'authentification
         if provider_data:
@@ -906,6 +949,11 @@ document.location.href="%s";
             login_help = open(interf_login).read()
         else:
             login_help = ""
+        if request_args.has_key('success_changepwd') and request_args['success_changepwd'][0] == '1':
+            login_help += "<span style='color: green'>"+_("Password changed with success")+"</span>"
+        if request_args.has_key('pwdneedchange') and request_args['pwdneedchange'][0] == '1':
+            if request_args.has_key('error') and request_args['error'][0] == '1':
+                login_help += "<span style='color: red'>"+_("Changing password failed, please try another password more secure")+"</span>"
         if config.DEBUG_LOG:
             autocomplete = 'on'
         else:
@@ -936,48 +984,66 @@ document.location.href="%s";
             provider_infos += '<div class="oidc_sep" style="clear:both;"></div>'
 
         # création de la page
-        content = AUTH_FORM % (provider_infos,
-                login_ticket,
-                service_input,
-                redirect_input,
-                _("Login"), _("Login"), autocomplete,
-                _("Password"), _("Password"), autocomplete,
-                passwd_check_func,
-                self.select_etab,
-                additional_form,
-                login_help,
-                _("Submit"),
-                avertissement_ie, avertissement)
-        # scripts dynamiques
-        javascript = """document.forms['cas_auth_form'].username.focus();setTimeout(callb_onload, 200);"""
-        header_script = """<script type="text/javascript" src="scripts/mootools-core-1.4.2.js"></script>
-<script type="text/javascript" src="scripts/tools.js?v=2.0"></script>
-<script type="text/javascript" src="scripts/etabs.js"></script>
-<script type="text/javascript" src="scripts/homonymes.js"></script>
-<script type="text/javascript">
-"""
-        otp_enabled = 'false'
-        if config.USE_SECURID:
-            # chargement des scripts pour gestion de la saisie de mots de passe OTP
-            otp_enabled = 'true'
-            header_script += open('interface/scripts/authform_otp.js').read() %\
-            (config.OTPPASS_MINSIZE,
-             config.OTPPASS_MAXSIZE,
-             config.OTPPASS_REGX)
-        header_script += open('interface/scripts/authform_ajax.js').read() %\
-        (otp_enabled,
-         _('label_password_OTP'),
-         _('label_registered'),
-         _('label_password'),
-         _('label_unregistered'),
-         _('label_password'),
-         _('select your origin'),
-         _('academic'),
-         _('several users correspond to '),
-         _('please select your authentication source'),
-)
-        header_script += open('interface/scripts/authform_ie.js').read().strip()
-        header_script += "\n</script>"
+        if request_args.has_key('pwdneedchange') and request_args['pwdneedchange'][0] == '1':
+            user = request_args['user'][0]
+            content = CHANGEPWD_FORM % (login_ticket,
+                    service_input,
+                    redirect_input,
+                    _("Login"),user,  _("Login"), autocomplete, user,
+                    _("New password"), _("New password"), autocomplete,
+                    passwd_check_func,
+                    login_help,
+                    _("Submit"))
+            header_script = """<script type="text/javascript" src="scripts/mootools-core-1.4.2.js"></script>
+             <script type="text/javascript" src="scripts/tools.js?v=2.0"></script>
+             <script type="text/javascript" src="scripts/homonymes.js"></script>
+             <script type="text/javascript" src="scripts/pwdchange.js"></script>"""
+            javascript = """document.forms['cas_changepwd_form'].username.focus();"""
+        else:
+            content = AUTH_FORM % (provider_infos,
+                    login_ticket,
+                    service_input,
+                    redirect_input,
+                    _("Login"), _("Login"), autocomplete,
+                    _("Password"), _("Password"), autocomplete,
+                    passwd_check_func,
+                    self.select_etab,
+                    additional_form,
+                    login_help,
+                    _("Submit"),
+                    avertissement_ie, avertissement)
+            # scripts dynamiques
+
+            header_script = """<script type="text/javascript" src="scripts/mootools-core-1.4.2.js"></script>
+    <script type="text/javascript" src="scripts/tools.js?v=2.0"></script>
+    <script type="text/javascript" src="scripts/etabs.js"></script>
+    <script type="text/javascript" src="scripts/homonymes.js"></script>
+    <script type="text/javascript">
+    """
+            otp_enabled = 'false'
+            if config.USE_SECURID:
+                # chargement des scripts pour gestion de la saisie de mots de passe OTP
+                otp_enabled = 'true'
+                header_script += open('interface/scripts/authform_otp.js').read() %\
+                (config.OTPPASS_MINSIZE,
+                config.OTPPASS_MAXSIZE,
+                config.OTPPASS_REGX)
+            header_script += open('interface/scripts/authform_ajax.js').read() %\
+            (otp_enabled,
+            _('label_password_OTP'),
+            _('label_registered'),
+            _('label_password'),
+            _('label_unregistered'),
+            _('label_password'),
+            _('select your origin'),
+            _('academic'),
+            _('several users correspond to '),
+            _('please select your authentication source'),
+    )
+            header_script += open('interface/scripts/authform_ie.js').read().strip()
+            header_script += "\n</script>"
+
+            javascript = """document.forms['cas_auth_form'].username.focus();setTimeout(callb_onload, 200);"""
         self.content = gen_page(self.auth_msg, content, css, javascript, header_script)
         return None
 
@@ -1046,6 +1112,10 @@ document.location.href="%s";
             password = unicode(request.args['password'][0],config.encoding)
         except:
             password = ""
+        try:
+            newpassword = request.args['newpassword'][0]
+        except KeyError:
+            newpassword = ''
         try:
             from_url = request.args['service'][0]
         except:
@@ -1077,9 +1147,9 @@ document.location.href="%s";
         except:
             # on n'est pas passé par /login : on renvoie dessus
             if from_url:
-                response = RedirectResponse('/login?service=%s' % from_url)
+                response = RedirectResponse('login?service=%s' % from_url)
             else:
-                response = RedirectResponse('/login')
+                response = RedirectResponse('login')
             return response
         session_infos = self.manager.login_sessions.get_session_info(ticket)
         # stockage des correspondances d'identifiants après retour d'une authentification externe
@@ -1131,6 +1201,17 @@ document.location.href="%s";
             response = RedirectResponse('/securid?%s' % urllib.urlencode(req_args))
             return response
         # on va tenter l'authentification sur tous les serveurs d'auth
+
+        if newpassword == '1':
+            cmd = '/usr/sbin/changepasswordeole.pl {} {}'.format(username, password)
+            try:
+                ret = subprocess.check_output(cmd, stderr=subprocess.STDOUT, shell=True)
+            except subprocess.CalledProcessError as e:
+                if from_url:
+                    return RedirectResponse('?service=%s&pwdneedchange=%s&user=%s&error=1' %(from_url, "1", username))
+                return RedirectResponse('?pwdneedchange=%s&user=%s&error=1' %("1", username))
+
+            return RedirectResponse('login?service=%s&success_changepwd=1' %(from_url))
         defer_auth = self.manager.authenticate(username, password, search_branch)
         return defer_auth.addCallback(self.callb_auth, from_url, redirect_to, cookies, provider_data)
 
@@ -1159,17 +1240,13 @@ document.location.href="%s";
             # on stocke un cookie dans le navigateur
             # chaine = '%s,%s'%(session_id), username.encode(config.encoding))
             session_id = session_id.encode(config.encoding)
-            cookies = []
-            for cookie in request_cookies:
-                if cookie.name != 'EoleSSOServer':
-                    cookies.append(cookie)
-            cookies.append(TwCookie("EoleSSOServer", session_id, path = "/", discard=True, secure=True))
+            cookies = self.manager.gen_session_cookies(session_id, request_cookies)
             if from_url == '':
                 # pas de service spécifié, on indique que la session est ouverte, ou on redirige si demandé
                 if redirect_to != '':
                     target_url = redirect_to
                 else:
-                    target_url = '/loggedin'
+                    target_url = 'loggedin'
             else:
                 # on redirige sur la page de l'application de destination
                 app_ticket = self.manager.get_app_ticket(session_id, from_url, from_credentials=True)
@@ -1189,9 +1266,21 @@ document.location.href="%s";
 
         # aucun des serveurs n'a authentifié l'utilisateur (meme l'authentification locale), on revient
         # sur la page de saisie de username/password
+        if 'error_ad' in user_data:
+            if from_url:
+                return RedirectResponse('?service=%s&error_ad=1&' %(from_url))
+            return RedirectResponse('?error_ad=1')
+
+        if user_data:
+            user = user_data['user']
+            pwdneedchange =  user_data['pwdneedchange']
+            if from_url:
+                return RedirectResponse('?service=%s&pwdneedchange=%s&user=%s' %(from_url, user_data['pwdneedchange'], user))
+            return RedirectResponse('?pwdneedchange=%s&user=%s' %(user_data['pwdneedchange'], user))
+
         if from_url:
-            return RedirectResponse('/?service=%s&failed=1' %(from_url))
-        return RedirectResponse('/?failed=1')
+            return RedirectResponse('?service=%s&failed=1' %(from_url))
+        return RedirectResponse('?failed=1')
 
     # XXX cette fonctionnalité n'a pas été retenue
     #def send_securid_request(self, request_id, username, password, sp_meta, req_args):
@@ -1240,7 +1329,7 @@ document.location.href="%s";
         if redirect_to == None:
             return self.set_headers(http.Response(stream=self.content))
         else:
-            if redirect_to != '/loggedin' and config.DEBUG_LOG:
+            if redirect_to != 'loggedin' and config.DEBUG_LOG:
                 log.msg("Authform, %s : %s" % (_("Redirecting to "), get_service_from_url(redirect_to)))
             return RedirectResponse(redirect_to)
 
@@ -1249,7 +1338,7 @@ document.location.href="%s";
         if segments[0] == 'logout':
             return saml_resources.SamlLogout(self.manager), ()
         # ERADICATION DES SESSIONS FANTOMES
-        user_session = getCookie(request, 'EoleSSOServer')
+        user_session = getCookie(request, config.COOKIE_NAME)
         if user_session and self.manager._DBCheckPendingLogout(user_session.value):
             log.msg(_("logout interrupted for session {0} : terminating").format(user_session.value.encode(config.encoding)))
             # REDIRECTION SUR LA PAGE DE LOGOUT
@@ -1291,11 +1380,16 @@ document.location.href="%s";
                 return SecuridCheck(self.manager), ()
             elif segments[0] == 'check_user_options':
                 return UserChecker(self.manager), ()
+            elif segments[0] == 'changepwd':
+                return LoginForm(self.manager, self.auth_server), ()
             # URL DE RECUPERATION DES ETABLISSEMENTS REPLIQUES (DISPATCHER)
             elif segments[0] == 'etabs':
                 return InfoEtabs(), ()
+            # URL DE RECUPERATION DES METRICS
             elif segments[0] == 'metrics' and metrics_enabled:
                 self.manager._METRICSCompute()
+                if self.manager.pushgateway:
+                    return MetricsPushgateway(self.manager),()
                 return MetricsResource(), ()
             # URLS UTILITAIRES
             elif segments[0] == 'favicon.ico':
